==> apache-fory-0.11.0-incubating-src.tar.gz: apache-fory-0.11.0-incubating-src/java/fory-core/src/test/java/org/apache/fory/resolver/DisallowedListTest.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.fory.resolver;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.rmi.server.UnicastRemoteObject;
import java.security.MessageDigest;
import java.util.Set;
import java.util.TreeSet;
import java.util.stream.Collectors;
import org.apache.fory.Fory;
import org.apache.fory.ForyTestBase;
import org.apache.fory.config.Language;
import org.apache.fory.exception.InsecureException;
import org.apache.fory.memory.Platform;
import org.apache.fory.reflect.ReflectionUtils;
import org.testng.Assert;
import org.testng.annotations.Test;

public class DisallowedListTest extends ForyTestBase {

  @Test
  public void testCalculateSHA256() throws Exception {
    final String disallowedListTxtPath =
        (String)
            ReflectionUtils.getDeclaredStaticFieldValue(
                DisallowedList.class, "DISALLOWED_LIST_TXT_PATH");
    try (InputStream is =
        DisallowedList.class.getClassLoader().getResourceAsStream(disallowedListTxtPath)) {
      assert is != null;
      Set<String> set =
          new BufferedReader(new InputStreamReader(is, StandardCharsets.UTF_8))
              .lines()
              .filter(line -> !line.isEmpty() && !line.startsWith("#"))
              .collect(Collectors.toSet());
      MessageDigest digest = MessageDigest.getInstance("SHA-256");
      byte[] hashBytes =
          digest.digest(String.join(",", new TreeSet<>(set)).getBytes(StandardCharsets.UTF_8));
      StringBuilder hexString = new StringBuilder();
      for (byte b : hashBytes) {
        String hex = Integer.toHexString(0xff & b);
        if (hex.length() == 1) {
          hexString.append('0');
        }
        hexString.append(hex);
      }
      System.out.println("SHA256 HASH for disallowed.txt is " + hexString);

      Assert.assertEquals(
          hexString.toString(),
          ReflectionUtils.getDeclaredStaticFieldValue(DisallowedList.class, "SHA256_HASH"),
          "Please update `DisallowedList#SHA256_HASH` with the above output hash value.");
    }
  }

  @Test
  public void testCheckHitDisallowedList() {
    // Hit the disallowed list.
    Assert.assertThrows(
        InsecureException.class,
        () -> DisallowedList.checkNotInDisallowedList("java.rmi.server.UnicastRemoteObject"));
    Assert.assertThrows(
        InsecureException.class,
        () ->
            DisallowedList.checkNotInDisallowedList(
                "com.sun.jndi.rmi.registry.BindingEnumeration"));
    Assert.assertThrows(
        InsecureException.class,
        () -> DisallowedList.checkNotInDisallowedList(java.beans.Expression.class.getName()));
    Assert.assertThrows(
        InsecureException.class,
        () -> DisallowedList.checkNotInDisallowedList(UnicastRemoteObject.class.getName()));

    // Not in the disallowed list.
    DisallowedList.checkNotInDisallowedList("java.util.HashMap");
  }

  @Test
  public void testSerializeDisallowedClass() {
    Fory[] allFory = new Fory[3];
    for (int i = 0; i < 3; i++) {
      boolean requireClassRegistration = i % 2 == 0;
      Fory fory =
          Fory.builder()
              .withLanguage(Language.JAVA)
              .requireClassRegistration(requireClassRegistration)
              .build();
      if (requireClassRegistration) {
        // Registered or unregistered Classes should be subject to disallowed list restrictions.
        fory.register(UnicastRemoteObject.class);
      }
      allFory[i] = fory;
    }

    for (Fory fory : allFory) {
      Assert.assertThrows(
          InsecureException.class,
          () -> fory.serialize(Platform.newInstance(UnicastRemoteObject.class)));
      serDe(fory, new String[] {"a", "b"});
    }
  }
}

==> apache-fory-0.11.1-incubating-src.tar.gz: apache-fory-0.11.1-incubating-src/java/fory-core/src/test/java/org/apache/fory/resolver/DisallowedListTest.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.fory.resolver;

import java.rmi.server.UnicastRemoteObject;
import java.util.Set;
import org.apache.fory.Fory;
import org.apache.fory.ForyTestBase;
import org.apache.fory.config.Language;
import org.apache.fory.exception.InsecureException;
import org.apache.fory.memory.Platform;
import org.testng.Assert;
import org.testng.annotations.Test;

public class DisallowedListTest extends ForyTestBase {

  @Test
  public void testDisallowedListNotEmpty() {
    Set<String> disallowedClasses = DisallowedList.getDisallowedClasses();
    Assert.assertFalse(disallowedClasses.isEmpty(), "Disallowed list should not be empty");
    Assert.assertTrue(
        disallowedClasses.size() > 200, "Disallowed list should contain many classes");
  }

  @Test
  public void testKnownDangerousClasses() {
    Set<String> disallowedClasses = DisallowedList.getDisallowedClasses();

    // Test some known dangerous classes are in the list
    Assert.assertTrue(disallowedClasses.contains("java.rmi.server.UnicastRemoteObject"));
    Assert.assertTrue(disallowedClasses.contains("com.sun.jndi.rmi.registry.BindingEnumeration"));
    Assert.assertTrue(disallowedClasses.contains("java.beans.Expression"));
    Assert.assertTrue(
        disallowedClasses.contains("org.apache.commons.collections.functors.InvokerTransformer"));
    Assert.assertTrue(disallowedClasses.contains("org.apache.xalan.xsltc.trax.TemplatesImpl"));
  }

  @Test
  public void testCheckHitDisallowedList() {
    // Hit the disallowed list.
    Assert.assertThrows(
        InsecureException.class,
        () -> DisallowedList.checkNotInDisallowedList("java.rmi.server.UnicastRemoteObject"));
    Assert.assertThrows(
        InsecureException.class,
        () ->
            DisallowedList.checkNotInDisallowedList(
                "com.sun.jndi.rmi.registry.BindingEnumeration"));
    Assert.assertThrows(
        InsecureException.class,
        () -> DisallowedList.checkNotInDisallowedList(java.beans.Expression.class.getName()));
    Assert.assertThrows(
        InsecureException.class,
        () -> DisallowedList.checkNotInDisallowedList(UnicastRemoteObject.class.getName()));

    // Not in the disallowed list.
    DisallowedList.checkNotInDisallowedList("java.util.HashMap");
  }

  @Test
  public void testSerializeDisallowedClass() {
    Fory[] allFory = new Fory[3];
    for (int i = 0; i < 3; i++) {
      boolean requireClassRegistration = i % 2 == 0;
      Fory fory =
          Fory.builder()
              .withLanguage(Language.JAVA)
              .requireClassRegistration(requireClassRegistration)
              .build();
      if (requireClassRegistration) {
        // Registered or unregistered Classes should be subject to disallowed list restrictions.
        fory.register(UnicastRemoteObject.class);
      }
      allFory[i] = fory;
    }

    for (Fory fory : allFory) {
      Assert.assertThrows(
          InsecureException.class,
          () -> fory.serialize(Platform.newInstance(UnicastRemoteObject.class)));
      serDe(fory, new String[] {"a", "b"});
    }
  }
}
